articles_types_list: list[str] = [
    "article",
    "Artículo",
    "journal-article",
    "Artículos de investigación",
    "Publicado en revista especializada",
    "Corto (Resumen)",
    "Revisión (Survey)",
    "Caso clínico",
    "Otro artículo publicado",
    "Artículos",
]
