institutions_list: list[str] = [
    "Archive",
    "Company",
    "Education",
    "Facility",
    "Government",
    "Healthcare",
    "Nonprofit",
    "Other",
    "archive",
    "company",
    "education",
    "facility",
    "government",
    "healthcare",
    "nonprofit",
    "other",
    "institution",
]
