external_urls_dict = {
    "doi": "https://doi.org/{id}",
    "lens": "https://www.lens.org/lens/scholar/article/{id}",
    "researchgate": "https://www.researchgate.net/profile/{id}",
    "orcid": "https://orcid.org/{id}",
    "scholar": "https://scholar.google.com/scholar?hl=en&as_sdt=0%2C5&q=info%3A{id}%3Ascholar.google.com&btnG=",
    "scholar citations": "https://scholar.google.com{id}",
    "scopus": "https://www.scopus.com/authid/detail.uri?authorId={id}",
    "openalex": "https://openalex.org/{id}",
    "ror": "https://ror.org/{id}",
    "wikidata": "https://www.wikidata.org/wiki/{id}",
    "isni": "https://isni.org/isni/{id}",
    "fundref": "https://search.crossref.org/search/funders?from_ui=yes&id={id}",
    "ucas": "https://www.ucas.com/explore/search/providers?query={id}",
    "linkedin": "https://www.linkedin.com/in/{id}",
    "ukprn": "https://www.ukrlp.co.uk/ukrlp/ukrlp_provider.page_pls_provDetails?x=&pv_status=VERIFIED&pv_vis_code=L&pn_p_id={id}",
    "wos": "https://publons.com/researcher/{id}",
}
