open_access_status_dict = {
    "gold": "Dorado",
    "closed": "Cerrado",
    "hybrid": "Híbrido",
    "diamond": "Diamante",
    "green": "Verde",
    "bronze": "Bronce",
    "no_info": "Sin información",
}
